
//Meteor.subscribe("acts");

/*global Iron*/

// session keys
var ALERT_KEY = "actDetailsAlert";
var ACT_TYPE_KEY = "actType";


// get info from the URL
var getState = function() {
    var routeName = Router.current().route.getName();
    var params = Router.current().getParams();
    return {
        routeName: routeName,
        newActType: params.newActType,
        actId: params.actId,
        isEditing: routeName==='actNew' || routeName==='actEdit'
    };
};


Template.actDetails.helpers({

    actFields: function () {
        return actFieldsForType;
    },

    state: function() {
        return getState();
    },

    caption: function() {
        switch(getState().routeName) {
            case "actNew":
                return "Setup new crawl";
            case "actView":
            case "actEdit":
                return "Crawl settings";
        }
    },

    alert: function() {
        return Session.get(ALERT_KEY);
    },

    act: function() {
        var act;
        var state = getState();
        if( state.routeName === 'actNew' ) {
            // creating new act
            // make sure it has a valid type
            if( !actFieldsForType[state.newActType] )
                return Router.go('pageNotFound', {code: 101});

            act = { type: state.newActType };

            // prefill values
            _.each( actFieldsForType[state.newActType], function( fieldSpec, fieldName ) {
                if( fieldSpec.prefillValue !== undefined )
                    act[fieldName] = fieldSpec.prefillValue;
            });

            // find a new unique customId
            var notUnique = {};
            Acts.find().forEach( function(act) {
                notUnique[act.customId] = true;
            });
            var prefix = "Untitled_", index = 0;
            do {
                act.customId = prefix + index++;
            }
            while( notUnique[act.customId] );

        } else {
            // showing existing act
            act = Acts.findOne(state.actId);
            if( !act ) {
                console.log("XXXX:" + JSON.stringify(state));
                return Router.go('pageNotFound', {code: 102});
            }
        }

        // load tables into session, where input_string_dict template will pick them up
        if( act.type === 'crawl' ) {
            Session.set("act_startUrls", act.startUrls);
            Session.set("act_crawlPurls", act.crawlPurls);
            Session.set("act_httpHeaders", act.httpHeaders);
        }

        Session.set(ACT_TYPE_KEY, act.type);
        Session.set(ALERT_KEY, null);

        return act;
    }
});


// shows Bootstrap alert message and scrolls to top
var showAlert = function(type, message, subMessages) {
    if( type && message ) {
        Session.set(ALERT_KEY, {type: type, message: message, subMessages: subMessages});
        if ($('#top').length)
            smoothScroll.animateScroll(null, '#top', {updateURL: false});
    } else
        Session.set(ALERT_KEY, null);
};


Template.actDetails.events({

    'click .edit_button': function () {
        Router.go('actEdit', {actId: getState().actId});
    },

    'click .save_button': function () {

        showAlert(null); // reset alert (if any)

        // get act type and state
        var actType = Session.get(ACT_TYPE_KEY);
        var actFields = actFieldsForType[actType];
        if( !actFields )
            return Router.go('pageNotFound', {code: 103});
        var state = getState();

        var act = {
            _id: state.actId,
            type: 'crawl',
            crawl: {}
        };

        // populate act with form data
        _.each( actFields, function( fieldSpec, fieldName ) {
            var htmlField = $("#act_" + fieldName);
            switch (fieldSpec.type) {
                case 'STRING':
                case 'JAVASCRIPT_CODE':
                    if( htmlField.length )
                        act[fieldName] = htmlField.val().trim();
                    break;

                case 'BOOLEAN':
                    if( htmlField.length )
                        act[fieldName] = htmlField.is(":checked");
                    break;

                case 'INTEGER':
                    if( htmlField.length ) {
                        var intVal = parseInt(htmlField.val());
                        if( !isNaN(intVal) )
                            act[fieldName] = intVal;
                    }
                    break;

                case 'STRING_KEY_VALUE_ARRAY':
                    var records = Session.get("act_" + fieldName);
                    // skip empty records
                    act[fieldName] = _.filter(records, function(rec){ return rec.key!=='' || rec.value!==''; });
                    break;

                default:
                    throw 'Da fooook: unexpected field type???';
            }
        });

        // check values
        // TODO: can add .has-error to inputs and make it better visually

        var validationErrors = [];

        var validationErrors = [];
        ['customId', 'startUrls', 'crawlPurls', 'httpHeaders'].forEach( function(fieldName) {
            var fieldSpec = actFields[fieldName];
            if( !Match.test(act[fieldName], fieldSpec.checkPattern) ) {
                validationErrors.push(fieldSpec.checkMessage || (fieldSpec.caption + " is not valid"));
                return;
            }
        });
        if( validationErrors.length > 0 ) {
            showAlert('danger', "Please fix the following problems:", validationErrors);
            return;
        }

        // TODO: check keyvalue array
        // TODO: maybe add special checks for CSS selectors, javascript, start URLs etc.

        // insert/update record
        Meteor.call("upsertAct", act, function(error, result) {
            if( error ) {
                if( error.error === 'act-customId-not-unique' )
                    showAlert('danger', "There is already another act with the same ID, please choose a unique ID.");
                else
                    showAlert('danger', "The record was not saved: " + error.toString());
                return;
            }
            Router.go('actView', {actId: result._id});
        });
    },

    'click .cancel_button': function () {
        showAlert(null);
        var state = getState();
        if( !state.actId )
            Router.go('actsList');
        else
            Router.go('actView', {actId: state.actId});
    },

    'click .back_button': function () {
        Router.go('actsList');
    }


});

